@@ -62,7 +62,7 @@ static void setup_panic_parse_options(Parse_Options opts)
 
 static inline void do_test(void)
 {
-#ifdef DEBUG
+#if 0
 	printf("%d\n", word_contains("said", PAST_TENSE_FORM_MARKER, dict));
 	printf("%d\n", word_contains("gave.v", PAST_TENSE_FORM_MARKER, dict));
 	printf("%d\n", word_contains("have", PAST_TENSE_FORM_MARKER, dict));
@@ -83,6 +83,9 @@ static void throwException(JNIEnv *env, const char* message)
 	exceptionClazz = (*env)->FindClass(env, "java/lang/RuntimeException");
 	if ((*env)->ThrowNew(env, exceptionClazz, msg) != 0)
 		(*env)->FatalError(env, "Fatal: link-grammar JNI: Cannot throw");
+
+	// I'm confused. Apparently Java exceptions return ??
+	free(msg);
 }
 
 // Note that we do NOT offer any kind of protection from having
