@@ -43,6 +43,7 @@
 
 #include "api-types.h"
 #include "corpus/corpus.h"
+#include "memory-pool.h"
 //#include "error.h"
 #include "utilities.h"
 
@@ -104,6 +105,13 @@ struct Parse_Options_s
 	bool display_morphology;/* if true, print morpho analysis of words */
 };
 
+typedef struct word_queue_s word_queue_t;
+struct word_queue_s
+{
+	Gword *word;
+	word_queue_t *next;
+};
+
 struct Sentence_s
 {
 	Dictionary  dict;           /* Words are defined from this dictionary */
@@ -111,16 +119,14 @@ struct Sentence_s
 	size_t length;              /* Number of words */
 	Word  *word;                /* Array of words after tokenization */
 	String_set *   string_set;  /* Used for assorted strings */
+	Pool_desc * fm_Match_node;  /* Fast-matcher Match_node memory pool */
+	Pool_desc * Table_connector_pool; /* Count memoizing memory pool */
 
-	/* Wordgraph stuff. FIXME: typedef for structs. */
+	/* Wordgraph stuff. FIXME: create stand-alone struct for these. */
 	Gword *wordgraph;            /* Tokenization wordgraph */
 	Gword *last_word;            /* FIXME Last issued word */
-	struct word_queue            /* Element in queue of words to tokenize */
-	{
-		Gword *word;
-		struct word_queue *next;
-	} *word_queue;
-	struct word_queue *word_queue_last;
+	word_queue_t *word_queue;    /* Element in queue of words to tokenize */
+	word_queue_t *word_queue_last;
 	size_t gword_node_num;       /* Debug - for differentiating between
 	                                wordgraph nodes with identical subwords. */
 
@@ -147,6 +153,7 @@ struct Sentence_s
 #ifdef USE_SAT_SOLVER
 	void *hook;                 /* Hook for the SAT solver */
 #endif /* USE_SAT_SOLVER */
+	void *disjuncts_connectors_memblock;
 };
 
 #endif
