@@ -11,31 +11,23 @@
 /*                                                                       */
 /*************************************************************************/
 
-#include <limits.h>
-#include <math.h>
 #include <string.h>
-#include <stdint.h>
 
 #include "api-structures.h"
-#include "connectors.h"  // for MAX_SENTENCE
 #include "corpus/corpus.h"
-#include "dict-common/dict-common.h"
 #include "dict-common/dict-utils.h" // for free_X_nodes
 #include "disjunct-utils.h"  // for free_disjuncts
-#include "error.h"
-#include "externs.h"
 #include "linkage/linkage.h"
+#include "memory-pool.h"
 #include "parse/histogram.h"  // for PARSE_NUM_OVERFLOW
 #include "parse/parse.h"
 #include "post-process/post-process.h" // for post_process_new()
-#include "print/print.h"
 #include "prepare/exprune.h"
+#include "string-set.h"
 #include "resources.h"
 #include "sat-solver/sat-encoder.h"
-#include "string-set.h"
 #include "tokenize/spellcheck.h"
 #include "tokenize/tokenize.h"
-#include "tokenize/tok-structures.h" // Needed for Gword_struct
 #include "tokenize/word-structures.h" // Needed for Word_struct/free_X_node
 #include "utilities.h"
 
@@ -105,7 +97,7 @@ Parse_Options parse_options_create(void)
 	Parse_Options po;
 
 	init_memusage();
-	po = (Parse_Options) xalloc(sizeof(struct Parse_Options_s));
+	po = (Parse_Options) malloc(sizeof(struct Parse_Options_s));
 
 	/* Here's where the values are initialized */
 
@@ -158,10 +150,10 @@ Parse_Options parse_options_create(void)
 	return po;
 }
 
-int parse_options_delete(Parse_Options  opts)
+int parse_options_delete(Parse_Options opts)
 {
 	resources_delete(opts->resources);
-	xfree(opts, sizeof(struct Parse_Options_s));
+	free(opts);
 	return 0;
 }
 
@@ -366,7 +358,7 @@ int parse_options_get_spell_guess(Parse_Options opts) {
 }
 
 void parse_options_set_short_length(Parse_Options opts, int short_length) {
-	opts->short_length = short_length;
+	opts->short_length = MIN(short_length, UNLIMITED_LEN);
 }
 
 int parse_options_get_short_length(Parse_Options opts) {
@@ -453,12 +445,13 @@ Sentence sentence_create(const char *input_string, Dictionary dict)
 {
 	Sentence sent;
 
-	sent = (Sentence) xalloc(sizeof(struct Sentence_s));
+	sent = (Sentence) malloc(sizeof(struct Sentence_s));
 	memset(sent, 0, sizeof(struct Sentence_s));
 
 	sent->dict = dict;
 	sent->string_set = string_set_create();
 	sent->rand_state = global_rand_state;
+	sent->disjuncts_connectors_memblock = NULL;
 
 	sent->postprocessor = post_process_new(dict->base_knowledge);
 
@@ -516,81 +509,31 @@ int sentence_split(Sentence sent, Parse_Options opts)
 
 static void free_sentence_words(Sentence sent)
 {
-	size_t i;
-
-	for (i = 0; i < sent->length; i++)
+	for (WordIdx i = 0; i < sent->length; i++)
 	{
 		free_X_nodes(sent->word[i].x);
-		free_disjuncts(sent->word[i].d);
 		free(sent->word[i].alternatives);
 	}
+	free_sentence_disjuncts(sent);
 	free((void *) sent->word);
 	sent->word = NULL;
 }
 
-// XXX FIXME ... these should find a home in the tokenize directory.
-static void wordgraph_delete(Sentence sent)
-{
-	Gword *w = sent->wordgraph;
-
-	while(NULL != w)
-	{
-		Gword *w_tofree = w;
-
-		free(w->prev);
-		free(w->next);
-		free(w->hier_position);
-		free(w->null_subwords);
-		w = w->chain_next;
-		free(w_tofree);
-	}
-	sent->wordgraph = sent->last_word = NULL;
-}
-
-static void word_queue_delete(Sentence sent)
-{
-	struct word_queue *wq = sent->word_queue;
-	while (NULL != wq)
-	{
-		struct word_queue *wq_tofree = wq;
-		wq = wq->next;
-		free(wq_tofree);
-	};
-	sent->word_queue = NULL;
-}
-
-/**
- * Delete the gword_set associated with the Wordgraph.
- * @w First Wordgraph word.
- */
-static void gword_set_delete(Gword *w)
-{
-	for (w = w->chain_next; NULL != w; w = w->chain_next)
-	{
-		gword_set *n;
-		for (gword_set *f = w->gword_set_head.chain_next; NULL != f; f = n)
-		{
-			n = f->chain_next;
-			free(f);
-		}
-	}
-}
-
 void sentence_delete(Sentence sent)
 {
 	if (!sent) return;
 	sat_sentence_delete(sent);
 	free_sentence_words(sent);
-	gword_set_delete(sent->wordgraph);
 	wordgraph_delete(sent);
-	word_queue_delete(sent);
 	string_set_delete(sent->string_set);
 	free_linkages(sent);
 	post_process_free(sent->postprocessor);
 	post_process_free(sent->constituent_pp);
 
 	global_rand_state = sent->rand_state;
-	xfree((char *) sent, sizeof(struct Sentence_s));
+	pool_delete(sent->fm_Match_node);
+	pool_delete(sent->Table_connector_pool);
+	free(sent);
 }
 
 int sentence_length(Sentence sent)
@@ -652,17 +595,6 @@ int sentence_link_cost(Sentence sent, LinkageIdx i)
 	return sent->lnkages[i].lifo.link_cost;
 }
 
-static void free_sentence_disjuncts(Sentence sent)
-{
-	size_t i;
-
-	for (i = 0; i < sent->length; ++i)
-	{
-		free_disjuncts(sent->word[i].d);
-		sent->word[i].d = NULL;
-	}
-}
-
 int sentence_parse(Sentence sent, Parse_Options opts)
 {
 	int rc;
@@ -678,6 +610,14 @@ int sentence_parse(Sentence sent, Parse_Options opts)
 		rc = sentence_split(sent, opts);
 		if (rc) return -1;
 	}
+	else
+	{
+		/* During a panic parse, we enter here a second time, with leftover
+		 * garbage. Free it. We really should make the code that is panicking
+		 * do this free, but right now, they have no API for it, so we do it
+		 * as a favor. XXX FIXME someday. */
+		free_sentence_disjuncts(sent);
+	}
 
 	/* Check for bad sentence length */
 	if (MAX_SENTENCE <= sent->length)
@@ -687,17 +627,12 @@ int sentence_parse(Sentence sent, Parse_Options opts)
 		return -2;
 	}
 
-	/* During a panic parse, we enter here a second time, with leftover
-	 * garbage. Free it. We really should make the code that is panicking
-	 * do this free, but right now, they have no PAI for it, so we do it
-	 * as a favor. XXX FIXME someday. */
-	free_sentence_disjuncts(sent);
 	resources_reset(opts->resources);
 
 	/* Expressions were set up during the tokenize stage.
 	 * Prune them, and then parse.
 	 */
-	expression_prune(sent);
+	expression_prune(sent, opts);
 	print_time(opts, "Finished expression pruning");
 	if (opts->use_sat_solver)
 	{
