@@ -33,7 +33,7 @@ void free_Exp(Exp * e)
 	if (e->type != CONNECTOR_type) {
 		free_E_list(e->u.l);
 	}
-	xfree((char *)e, sizeof(Exp));
+	free(e);
 }
 
 void free_E_list(E_list * l)
@@ -41,7 +41,7 @@ void free_E_list(E_list * l)
 	if (l == NULL) return;
 	free_E_list(l->next);
 	free_Exp(l->e);
-	xfree((char *)l, sizeof(E_list));
+	free(l);
 }
 
 /* Returns the number of connectors in the expression e */
@@ -65,7 +65,7 @@ Exp * copy_Exp(Exp * e)
 {
 	Exp * n;
 	if (e == NULL) return NULL;
-	n = (Exp *) xalloc(sizeof(Exp));
+	n = malloc(sizeof(Exp));
 	*n = *e;
 	if (e->type != CONNECTOR_type) {
 		n->u.l = copy_E_list(e->u.l);
@@ -77,16 +77,16 @@ static E_list * copy_E_list(E_list * l)
 {
 	E_list * nl;
 	if (l == NULL) return NULL;
-	nl = (E_list *) xalloc(sizeof(E_list));
+	nl = malloc(sizeof(E_list));
 	nl->next = copy_E_list(l->next);
 	nl->e = copy_Exp(l->e);
 	return nl;
 }
 
 /**
- * Compare two expressions, return 1 for equal, 0 for unequal
+ * Compare two expressions, return true for equal, false for unequal
  */
-static int exp_compare(Exp * e1, Exp * e2)
+static bool exp_compare(Exp * e1, Exp * e2)
 {
 	E_list *el1, *el2;
 
@@ -102,8 +102,8 @@ static int exp_compare(Exp * e1, Exp * e2)
 	{
 		if (e1->dir != e2->dir)
 			return 0;
-		/* printf("%s %s\n",e1->u.string,e2->u.string); */
-		if (!string_set_cmp(e1->u.string, e2->u.string))
+		/* printf("%s %s\n",e1->u.condesc->string,e2->u.condesc->string); */
+		if (e1->u.condesc != e2->u.condesc)
 			return 0;
 	}
 	else
@@ -116,12 +116,10 @@ static int exp_compare(Exp * e1, Exp * e2)
 			if ((el1==NULL)||(el2==NULL))
 				return 0;
 			/* fail if they are not compared */
-			if (exp_compare(el1->e, el2->e) == 0)
+			if (!exp_compare(el1->e, el2->e))
 				return 0;
-			if (el1!=NULL)
-				el1 = el1->next;
-			if (el2!=NULL)
-				el2 = el2->next;
+			el1 = el1->next;
+			el2 = el2->next;
 		}
 	}
 	return 1; /* if never returned 0, return 1 */
@@ -143,7 +141,7 @@ static int exp_contains(Exp * super, Exp * sub)
 
 	if (sub==NULL || super==NULL)
 		return 0;
-	if (exp_compare(sub,super)==1)
+	if (exp_compare(sub,super))
 		return 1;
 	if (super->type==CONNECTOR_type)
 	  return 0; /* super is a leaf */
@@ -168,7 +166,7 @@ void free_X_nodes(X_node * x)
 	for (; x!= NULL; x = y) {
 		y = x->next;
 		free_Exp(x->exp);
-		xfree((char *)x, sizeof(X_node));
+		free(x);
 	}
 }
 
@@ -236,8 +234,8 @@ static bool exp_has_connector(const Exp * e, int depth, const char * cs,
 	if (e->type == CONNECTOR_type)
 	{
 		if (direction != e->dir) return false;
-		return smart_match ? easy_match(e->u.string, cs)
-		                   : string_set_cmp(e->u.string, cs);
+		return smart_match ? easy_match(e->u.condesc->string, cs)
+		                   : string_set_cmp(e->u.condesc->string, cs);
 	}
 
 	if (depth == 0) return false;
@@ -280,7 +278,7 @@ const char * word_only_connector(Dict_node * dn)
 {
 	Exp * e = dn->exp;
 	if (CONNECTOR_type == e->type)
-		return e->u.string;
+		return e->u.condesc->string;
 	return NULL;
 }
 
