@@ -11,6 +11,7 @@
 /*                                                                       */
 /*************************************************************************/
 
+#include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>   /* for fstat() */
 
@@ -76,7 +77,9 @@ char * join_path(const char * prefix, const char * suffix)
 
 /* global - but that's OK, since this is set only during initialization,
  * and is is thenceforth a read-only item. So it doesn't need to be
- * locked.
+ * locked.  XXX Assuming the user actually does set it only once,
+ * and doesn't hammer on this inside of a thread loop, in which case
+ * it will race and abort with a double-free error.
  */
 static char * custom_data_dir = NULL;
 
@@ -172,7 +175,8 @@ void * object_open(const char *filename,
                    void * (*opencb)(const char *, const void *),
                    const void * user_data)
 {
-	static char *path_found; /* directory path cache */
+	/* directory path cache -- per-thread storage. */
+	static TLS char *path_found;
 	char *completename = NULL;
 	void *fp = NULL;
 	char *data_dir = NULL;
@@ -180,9 +184,10 @@ void * object_open(const char *filename,
 
 	if (NULL == filename)
 	{
-		/* Invalidate the directory path cache */
-		free(path_found);
+		/* Invalidate the directory path cache. */
+		char *pf = path_found;
 		path_found = NULL;
+		free(pf);
 		return NULL;
 	}
 
@@ -249,16 +254,15 @@ void * object_open(const char *filename,
 	}
 	else if (NULL == path_found)
 	{
-		size_t i;
-
-		path_found = strdup((NULL != completename) ? completename : filename);
+		char *pfnd = strdup((NULL != completename) ? completename : filename);
 		if (0 < verbosity)
-			prt_error("Info: Dictionary found at %s\n", path_found);
-		for (i = 0; i < 2; i++)
+			prt_error("Info: Dictionary found at %s\n", pfnd);
+		for (size_t i = 0; i < 2; i++)
 		{
-			char *root = strrchr(path_found, DIR_SEPARATOR[0]);
+			char *root = strrchr(pfnd, DIR_SEPARATOR[0]);
 			if (NULL != root) *root = '\0';
 		}
+		path_found = pfnd;
 	}
 
 	free(data_dir);
@@ -312,9 +316,9 @@ char *get_file_contents(const char * dict_name)
 {
 	int fd;
 	size_t tot_size;
-	int left;
+	size_t tot_read = 0;
 	struct stat buf;
-	char * contents, *p;
+	char * contents;
 
 	/* On Windows, 'b' (binary mode) is mandatory, otherwise fstat file length
 	 * is confused by crlf counted as one byte. POSIX systems just ignore it. */
@@ -330,29 +334,39 @@ char *get_file_contents(const char * dict_name)
 
 	contents = (char *) malloc(sizeof(char) * (tot_size+7));
 
-	/* Now, read the whole file. */
-	p = contents;
-	*p = '\0';
-	left = tot_size + 7;
+	/* Now, read the whole file.
+	 * Normally, only a single call of fread() is needed. */
 	while (1)
 	{
-		char *rv = fgets(p, left, fp);
-		if (NULL == rv || feof(fp))
+		size_t read_size = fread(contents, 1, tot_size+7, fp);
+
+		if (0 == read_size)
+		{
+			bool err = (0 != ferror(fp));
+			fclose(fp);
+
+			if (err)
+			{
+				char errbuf[64];
+
+				strerror_r(errno, errbuf, sizeof(errbuf));
+				prt_error("Error: %s: Read error (%s)\n", dict_name, errbuf);
+				free(contents);
+				return NULL;
+			}
 			break;
-		while (*p != '\0') { p++; left--; }
-		if (left < 0)
-			 break;
+		}
+		tot_read += read_size;
 	}
 
-	fclose(fp);
-
-	if (left < 0)
+	if (tot_read > tot_size+6)
 	{
-		prt_error("Error: File size is insane!\n");
+		prt_error("Error: %s: File size is insane (%zu)!\n", dict_name, tot_size);
 		free(contents);
 		return NULL;
 	}
 
+	contents[tot_read] = '\0';
 	return contents;
 }
 
