@@ -149,52 +149,38 @@ static const char * build_idiom_word_name(Dictionary dict, const char * s)
  * correct names (with .Ix suffixes).
  * The list is reversed from the way they occur in the string.
  * A pointer to this list is returned.
+ * This function is called after is_idiom_string() ensures the validity
+ * of the given string.
  */
 static Dict_node * make_idiom_Dict_nodes(Dictionary dict, const char * string)
 {
-	Dict_node * dn, * dn_new;
-	char * t, *s, *p;
-	bool more;
-	unsigned int sz;
-	dn = NULL;
+	Dict_node * dn = NULL;
+	char * s = strdupa(string);
+	const char * t;
 
-	sz = strlen(string)+1;
-	p = s = (char *) xalloc(sz);
-	strcpy(s, string);
-
-	while (*s != '\0') {
-		t = s;
-		while ((*s != '\0') && (*s != '_')) s++;
-		if (*s == '_') {
-			more = true;
-			*s = '\0';
-		} else {
-			more = false;
-		}
-		dn_new = (Dict_node *) xalloc(sizeof (Dict_node));
+	for (t = s; NULL != s; t = s)
+	{
+		s = strchr(s, '_');
+		if (NULL != s) *s++ = '\0';
+		Dict_node *dn_new = (Dict_node *) malloc(sizeof (Dict_node));
 		dn_new->right = dn;
 		dn = dn_new;
 		dn->string = string_set_add(t, dict->string_set);
 		dn->file = NULL;
-		if (more) s++;
 	}
 
-	xfree(p, sz);
 	return dn;
 }
 
-static char current_name[] = "AAAAAAAA";
-#define CN_size (sizeof(current_name)-1)
-
-static void increment_current_name(void)
+static void increment_current_name(Dictionary dict)
 {
-	int i = CN_size-1;
+	int i = IDIOM_LINK_SZ-2;
 
 	do
 	{
-		current_name[i]++;
-		if (current_name[i] <= 'Z') return;
-		current_name[i] = 'A';
+		dict->current_idiom[i]++;
+		if (dict->current_idiom[i] <= 'Z') return;
+		dict->current_idiom[i] = 'A';
 	} while (i-- > 0);
 	assert(0, "increment_current_name: Overflow");
 }
@@ -206,11 +192,11 @@ static void increment_current_name(void)
  */
 static const char * generate_id_connector(Dictionary dict)
 {
-	char buff[2*MAX_WORD];
+	char buff[IDIOM_LINK_SZ+4];
 	unsigned int i;
 	char * t;
 
-	for (i=0; current_name[i] == 'A'; i++)
+	for (i=0; dict->current_idiom[i] == 'A'; i++)
 	  ;
 	/* i is now the number of characters of current_name to skip */
 	t = buff;
@@ -218,9 +204,9 @@ static const char * generate_id_connector(Dictionary dict)
 	/* All idiom connector names start with the two letters "ID" */
 	*t++ = 'I';
 	*t++ = 'D';
-	for (; i < CN_size; i++ )
+	for (; i < IDIOM_LINK_SZ; i++ )
 	{
-		*t++ = current_name[i] ;
+		*t++ = dict->current_idiom[i] ;
 	}
 	*t++ = '\0';
 	return string_set_add(buff, dict->string_set);
@@ -253,13 +239,13 @@ void insert_idiom(Dictionary dict, Dict_node * dn)
 		          "\tThis word will be ignored\n",
 		          s, dict->line_number);
 
-		xfree((char *)dn, sizeof (Dict_node));
+		free(dn);
 		return;
 	}
 
 	dn_list = start_dn_list = make_idiom_Dict_nodes(dict, s);
 
-	xfree((char *)dn, sizeof (Dict_node));
+	free(dn);
 	dn = NULL;
 
 	assert(dn_list->right != NULL, "Idiom string with only one connector");
@@ -269,15 +255,15 @@ void insert_idiom(Dictionary dict, Dict_node * dn)
 
 	/* ----- this code just sets up the node fields of the dn_list ----*/
 	nc = Exp_create(&dict->exp_list);
-	nc->u.string = generate_id_connector(dict);
+	nc->u.condesc = condesc_add(&dict->contable, generate_id_connector(dict));
 	nc->dir = '-';
 	nc->multi = false;
 	nc->type = CONNECTOR_type;
 	nc->cost = 0;
 
 	n1 = Exp_create(&dict->exp_list);
-	n1->u.l = ell = (E_list *) xalloc(sizeof(E_list));
-	ell->next = elr = (E_list *) xalloc(sizeof(E_list));
+	n1->u.l = ell = (E_list *) malloc(sizeof(E_list));
+	ell->next = elr = (E_list *) malloc(sizeof(E_list));
 	elr->next = NULL;
 	ell->e = nc;
 	elr->e = no;
@@ -293,25 +279,24 @@ void insert_idiom(Dictionary dict, Dict_node * dn)
 		/* generate the expression for a middle idiom word */
 
 		n1 = Exp_create(&dict->exp_list);
-		n1->u.string = NULL;
 		n1->type = AND_type;
 		n1->cost = 0;
-		n1->u.l = ell = (E_list *) xalloc(sizeof(E_list));
-		ell->next = elr = (E_list *) xalloc(sizeof(E_list));
+		n1->u.l = ell = (E_list *) malloc(sizeof(E_list));
+		ell->next = elr = (E_list *) malloc(sizeof(E_list));
 		elr->next = NULL;
 
 		nc = Exp_create(&dict->exp_list);
-		nc->u.string = generate_id_connector(dict);
+		nc->u.condesc = condesc_add(&dict->contable, generate_id_connector(dict));
 		nc->dir = '+';
 		nc->multi = false;
 		nc->type = CONNECTOR_type;
 		nc->cost = 0;
 		elr->e = nc;
 
-		increment_current_name();
+		increment_current_name(dict);
 
 		nc = Exp_create(&dict->exp_list);
-		nc->u.string = generate_id_connector(dict);
+		nc->u.condesc = condesc_add(&dict->contable, generate_id_connector(dict));
 		nc->dir = '-';
 		nc->multi = false;
 		nc->type = CONNECTOR_type;
@@ -326,7 +311,7 @@ void insert_idiom(Dictionary dict, Dict_node * dn)
 	/* now generate the last one */
 
 	nc = Exp_create(&dict->exp_list);
-	nc->u.string = generate_id_connector(dict);
+	nc->u.condesc = condesc_add(&dict->contable, generate_id_connector(dict));
 	nc->dir = '+';
 	nc->multi = false;
 	nc->type = CONNECTOR_type;
@@ -334,7 +319,7 @@ void insert_idiom(Dictionary dict, Dict_node * dn)
 
 	dn_list->exp = nc;
 
-	increment_current_name();
+	increment_current_name(dict);
 
 	/* ---- end of the code alluded to above ---- */
 
@@ -351,7 +336,6 @@ void insert_idiom(Dictionary dict, Dict_node * dn)
 		dict->num_entries++;
 		dn_list = xdn;
 	}
-	/* xfree((char *)s, s_length+1); strings are handled by string_set */
 }
 
 /**
