@@ -77,7 +77,7 @@ static void load_affix(Dictionary afdict, Dict_node *dn, int l)
 		               /*notify_err*/true), string);
 
 		dnx = dn->left;
-		xfree((char *)dn, sizeof(Dict_node));
+		free(dn);
 	}
 }
 
@@ -114,7 +114,7 @@ dictionary_six_str(const char * lang,
 	const char * t;
 	Dictionary dict;
 
-	dict = (Dictionary) xalloc(sizeof(struct Dictionary_s));
+	dict = (Dictionary) malloc(sizeof(struct Dictionary_s));
 	memset(dict, 0, sizeof(struct Dictionary_s));
 
 	/* Language and file-name stuff */
@@ -125,6 +125,9 @@ dictionary_six_str(const char * lang,
 	lgdebug(D_USER_FILES, "Debug: Language: %s\n", dict->lang);
 	dict->name = string_set_add(dict_name, dict->string_set);
 
+	memset(dict->current_idiom, 'A', IDIOM_LINK_SZ-1);
+	dict->current_idiom[IDIOM_LINK_SZ-1] = 0;
+
 	/*
 	 * A special setup per dictionary type. The check here assumes the affix
 	 * dictionary name contains "affix". FIXME: For not using this
@@ -145,6 +148,7 @@ dictionary_six_str(const char * lang,
 		dict->lookup_wild = file_lookup_wild;
 		dict->free_lookup = free_llist;
 		dict->lookup = file_boolean_lookup;
+		dict->contable.num_con = 1<<13;
 	}
 	else
 	{
@@ -154,9 +158,14 @@ dictionary_six_str(const char * lang,
 		afclass_init(dict);
 		dict->insert_entry = load_affix;
 		dict->lookup = return_true;
+		dict->contable.num_con = 1<<9;
 	}
 	dict->affix_table = NULL;
 
+	dict->contable.size = 0;
+	dict->contable.length_limit_def = NULL;
+	dict->contable.length_limit_def_next = &dict->contable.length_limit_def;
+
 	/* Read dictionary from the input string. */
 	dict->input = input;
 	dict->pin = dict->input;
@@ -208,6 +217,7 @@ dictionary_six_str(const char * lang,
 	if (compile_regexs(dict->regex_root, dict))
 	{
 		locale = setlocale(LC_CTYPE, locale);         /* Restore the locale. */
+		assert(NULL != locale, "Cannot restore program locale");
 		goto failure;
 	}
 	locale = setlocale(LC_CTYPE, locale);            /* Restore the locale. */
@@ -220,6 +230,7 @@ dictionary_six_str(const char * lang,
 	dict->base_knowledge  = pp_knowledge_open(pp_name);
 	dict->hpsg_knowledge  = pp_knowledge_open(cons_name);
 
+	sort_condesc_by_uc_constring(dict);
 	dictionary_setup_defines(dict);
 
 	// Special-case hack.
