@@ -128,6 +128,14 @@ static void dict_error2(Dictionary dict, const char * s, const char *s2)
 	if (dict->recursive_error) return;
 	dict->recursive_error = true;
 
+	char token[MAX_TOKEN_LENGTH];
+	strcpy(token, dict->token);
+	bool save_is_special    = dict->is_special;
+	const char * save_input = dict->input;
+	const char * save_pin   = dict->pin;
+	int save_already_got_it = dict->already_got_it;
+	int save_line_number    = dict->line_number;
+
 	tokens[0] = '\0';
 	for (i=0; i<5 && dict->token[0] != '\0' ; i++)
 	{
@@ -137,16 +145,23 @@ static void dict_error2(Dictionary dict, const char * s, const char *s2)
 	}
 	tokens[pos] = '\0';
 
+	strcpy(dict->token, token);
+	dict->is_special     = save_is_special;
+	dict->input          = save_input;
+	dict->pin            = save_pin;
+	dict->already_got_it = save_already_got_it;
+	dict->line_number    = save_line_number;
+
 	if (s2)
 	{
 		prt_error("Error: While parsing dictionary %s:\n"
-		          "%s %s\n\t line %d, tokens = %s\n\\",
+		          "%s %s\n\t Line %d, next tokens: %s\n",
 		          dict->name, s, s2, dict->line_number, tokens);
 	}
 	else
 	{
 		prt_error("Error: While parsing dictionary %s:\n"
-		          "%s\n\t line %d, tokens = %s\n\\",
+		          "%s\n\t Line %d, next tokens: %s\n",
 		          dict->name, s, dict->line_number, tokens);
 	}
 	dict->recursive_error = false;
@@ -169,10 +184,10 @@ static void warning(Dictionary dict, const char * s)
  * If we're in quote mode, it does not consider the % character for
  * comments.   Note that the returned character is a wide character!
  */
-typedef char* utf8char;
-static utf8char get_character(Dictionary dict, int quote_mode)
+#define MAXUTFLEN 7
+typedef char utf8char[MAXUTFLEN];
+static bool get_character(Dictionary dict, int quote_mode, utf8char uc)
 {
-	static char uc[7];
 	int i = 0;
 
 	while (1)
@@ -182,6 +197,16 @@ static utf8char get_character(Dictionary dict, int quote_mode)
 		/* Skip over all comments */
 		if ((c == '%') && (!quote_mode))
 		{
+			if (0 == strncmp(dict->pin, SUPPRESS, sizeof(SUPPRESS)-1))
+			{
+				const char *nl = strchr(dict->pin + sizeof(SUPPRESS)-1, '\n');
+				if (NULL != nl)
+				{
+					dict->suppress_warning =
+						strndup(dict->pin + sizeof(SUPPRESS)-1,
+					           nl - dict->pin - sizeof(SUPPRESS) + 1);
+				}
+			}
 			while ((c != 0x0) && (c != '\n')) c = *(dict->pin++);
 			dict->line_number++;
 			continue;
@@ -196,12 +221,12 @@ static utf8char get_character(Dictionary dict, int quote_mode)
 		{
 			uc[0] = c;
 			uc[1] = 0x0;
-			return uc;
+			return true;
 		}
 
 		uc[0] = c;
 		i = 1;
-		while (i < 6)
+		while (i < MAXUTFLEN-1)
 		{
 			c = *(dict->pin++);
 			/* If we're onto the next char, we're done. */
@@ -215,10 +240,10 @@ static utf8char get_character(Dictionary dict, int quote_mode)
 			i++;
 		}
 		dict_error(dict, "UTF8 char is too long");
-		return NULL;
+		return false;
 	}
 	uc[0] = 0x0;
-	return uc;
+	return true;
 }
 
 
@@ -264,7 +289,7 @@ static bool link_advance(Dictionary dict)
 		if (dict->already_got_it == EOF) {
 			dict->token[0] = '\0';
 		} else {
-			dict->token[0] = dict->already_got_it; /* specials are one byte */
+			dict->token[0] = (char)dict->already_got_it; /* specials are one byte */
 			dict->token[1] = '\0';
 		}
 		dict->already_got_it = '\0';
@@ -273,8 +298,8 @@ static bool link_advance(Dictionary dict)
 
 	do
 	{
-		c = get_character(dict, false);
-		if (NULL == c) return false;
+		bool ok = get_character(dict, false, c);
+		if (!ok) return false;
 	}
 	while (lg_isspace(c[0]));
 
@@ -293,7 +318,6 @@ static bool link_advance(Dictionary dict)
 			    (*dict->pin == ':' || *dict->pin == ';' ||
 			    lg_isspace(*dict->pin))) {
 
-				quote_mode = false;
 				dict->token[i] = '\0';
 				return true;
 			}
@@ -338,10 +362,10 @@ static bool link_advance(Dictionary dict)
 				while (c[nr]) {dict->token[i] = c[nr]; i++; nr++; }
 			}
 		}
-		c = get_character(dict, quote_mode);
-		if (NULL == c) return false;
+		bool ok = get_character(dict, quote_mode, c);
+		if (!ok) return false;
 	}
-	return true;
+	/* unreachable */
 }
 
 /**
@@ -446,7 +470,7 @@ int dict_order_strict(char *s, char *t)
 
 /* terse version */
 /* If one word contains a dot, the other one must also! */
-static inline int dict_order_strict(const char *s, Dict_node * dn)
+static inline int dict_order_strict(const char *s, const Dict_node * dn)
 {
 	const char * t = dn->string;
 	while (*s != '\0' && *s == *t) {s++; t++;}
@@ -496,7 +520,7 @@ static inline int dict_order_bare(const char *s, const Dict_node * dn)
  * you come to the end of one of them, or until you find unequal
  * characters.  A "*" matches anything before the subscript mark.
  * Otherwise, replace SUBSCRIPT_MARK by "\0", and take the difference.
- * his behavior matches that of the function dict_order_bare().
+ * This behavior matches that of the function dict_order_bare().
  */
 #define D_DOW 6
 static inline int dict_order_wild(const char * s, const Dict_node * dn)
@@ -517,48 +541,27 @@ static inline int dict_order_wild(const char * s, const Dict_node * dn)
 /**
  * dict_match --  return true if strings match, else false.
  * A "bare" string (one without a subscript) will match any corresponding
- * string with a suffix; so, for example, "make" and "make.n" are
+ * string with a subscript; so, for example, "make" and "make.n" are
  * a match.  If both strings have subscripts, then the subscripts must match.
  *
  * A subscript is the part that follows the SUBSCRIPT_MARK.
  */
 static bool dict_match(const char * s, const char * t)
 {
-	char *ds, *dt;
-	ds = strrchr(s, SUBSCRIPT_MARK);
-	dt = strrchr(t, SUBSCRIPT_MARK);
+	while ((*s != '\0') && (*s == *t)) { s++; t++; }
 
-#if SUBSCRIPT_MARK == '.'
-	/* a dot at the end or a dot followed by a number is NOT
-	 * considered a subscript */
-	if ((dt != NULL) && ((*(dt+1) == '\0') ||
-	    (isdigit((int)*(dt+1))))) dt = NULL;
-	if ((ds != NULL) && ((*(ds+1) == '\0') ||
-	    (isdigit((int)*(ds+1))))) ds = NULL;
-#endif
+	if (*s == *t) return true; /* both are '\0' */
+	if ((*s == 0) && (*t == SUBSCRIPT_MARK)) return true;
+	if ((*s == SUBSCRIPT_MARK) && (*t == 0)) return true;
 
-	/* dt is NULL when there's no prefix ... */
-	if (dt == NULL && ds != NULL) {
-		if (((int)strlen(t)) > (ds-s)) return false;   /* we need to do this to ensure that */
-		return (strncmp(s, t, ds-s) == 0);             /* "i.e." does not match "i.e" */
-	} else if (dt != NULL && ds == NULL) {
-		if (((int)strlen(s)) > (dt-t)) return false;
-		return (strncmp(s, t, dt-t) == 0);
-	} else {
-		return (strcmp(s, t) == 0);
-	}
+	return false;
 }
 
 /* ======================================================================== */
 
 static inline Dict_node * dict_node_new(void)
 {
-	return (Dict_node*) xalloc(sizeof(Dict_node));
-}
-
-static inline void free_dict_node(Dict_node *dn)
-{
-	xfree((char *)dn, sizeof(Dict_node));
+	return (Dict_node*) malloc(sizeof(Dict_node));
 }
 
 /**
@@ -583,7 +586,7 @@ static Dict_node * prune_lookup_list(Dict_node *llist, const char * s)
 		}
 		else
 		{
-			free_dict_node(dn);
+			free(dn);
 		}
 	}
 
@@ -685,7 +688,7 @@ void file_free_lookup(Dict_node *llist)
 	while (llist != NULL)
 	{
 		n = llist->right;
-		free_dict_node(llist);
+		free(llist);
 		llist = n;
 	}
 }
@@ -696,7 +699,7 @@ void free_insert_list(Dict_node *ilist)
 	while (ilist != NULL)
 	{
 		n = ilist->left;
-		free_dict_node(ilist);
+		free(ilist);
 		ilist = n;
 	}
 }
@@ -746,32 +749,47 @@ static Dict_node * abridged_lookup_list(const Dictionary dict, const char *s)
 	return llist;
 }
 
+/**
+ * strict_lookup_list() - return exact match in the dictionary
+ *
+ * Returns a pointer to a lookup list of the words in the dictionary.
+ * Excludes any idioms that contain the word.
+ *
+ * This list is made up of Dict_nodes, linked by their right pointers.
+ * The node, file and string fields are copied from the dictionary.
+ *
+ * The list normally has 0 or 1 elements, unless the given word
+ * appears more than once in the dictionary.
+ *
+ * The returned list must be freed with file_free_lookup().
+ */
+static Dict_node * strict_lookup_list(const Dictionary dict, const char *s)
+{
+	Dict_node *llist;
+	llist = rdictionary_lookup(NULL, dict->root, s, false, dict_order_strict);
+	llist = prune_lookup_list(llist, s);
+	return llist;
+}
+
 /* ======================================================================== */
 /**
  * Allocate a new Exp node and link it into the exp_list for freeing later.
  */
 Exp * Exp_create(Exp_list *eli)
 {
-	Exp * e;
-	e = (Exp *) xalloc(sizeof(Exp));
+	Exp * e = malloc(sizeof(Exp));
 	e->next = eli->exp_list;
 	eli->exp_list = e;
 	return e;
 }
 
-static inline void exp_free(Exp * e)
-{
-	xfree((char *)e, sizeof(Exp));
-}
-
 /**
  * This creates a node with zero children.  Initializes
  * the cost to zero.
  */
 static Exp * make_zeroary_node(Exp_list * eli)
 {
-	Exp * n;
-	n = Exp_create(eli);
+	Exp * n = Exp_create(eli);
 	n->type = AND_type;  /* these must be AND types */
 	n->cost = 0.0;
 	n->u.l = NULL;
@@ -788,7 +806,7 @@ static Exp * make_unary_node(Exp_list * eli, Exp * e)
 	n = Exp_create(eli);
 	n->type = AND_type;  /* these must be AND types */
 	n->cost = 0.0;
-	n->u.l = (E_list *) xalloc(sizeof(E_list));
+	n->u.l = (E_list *) malloc(sizeof(E_list));
 	n->u.l->next = NULL;
 	n->u.l->e = e;
 	return n;
@@ -807,8 +825,8 @@ static Exp * make_and_node(Exp_list * eli, Exp* nl, Exp* nr)
 	n->type = AND_type;
 	n->cost = 0.0;
 
-	n->u.l = ell = (E_list *) xalloc(sizeof(E_list));
-	ell->next = elr = (E_list *) xalloc(sizeof(E_list));
+	n->u.l = ell = (E_list *) malloc(sizeof(E_list));
+	ell->next = elr = (E_list *) malloc(sizeof(E_list));
 	elr->next = NULL;
 
 	ell->e = nl;
@@ -829,8 +847,8 @@ static Exp * make_or_node(Exp_list *eli, Exp* nl, Exp* nr)
 	n->type = OR_type;
 	n->cost = 0.0;
 
-	n->u.l = ell = (E_list *) xalloc(sizeof(E_list));
-	ell->next = elr = (E_list *) xalloc(sizeof(E_list));
+	n->u.l = ell = (E_list *) malloc(sizeof(E_list));
+	ell->next = elr = (E_list *) malloc(sizeof(E_list));
 	elr->next = NULL;
 
 	ell->e = nl;
@@ -857,18 +875,24 @@ static Exp * make_optional_node(Exp_list *eli, Exp * e)
 static Exp * make_dir_connector(Dictionary dict, int i)
 {
 	Exp* n = Exp_create(&dict->exp_list);
+	char *constring;
+
 	n->dir = dict->token[i];
 	dict->token[i] = '\0';   /* get rid of the + or - */
 	if (dict->token[0] == '@')
 	{
-		n->u.string = string_set_add(dict->token+1, dict->string_set);
+		constring = dict->token+1;
 		n->multi = true;
 	}
 	else
 	{
-		n->u.string = string_set_add(dict->token, dict->string_set);
+		constring = dict->token;
 		n->multi = false;
 	}
+
+	n->u.condesc = condesc_add(&dict->contable,
+	                            string_set_add(constring, dict->string_set));
+	if (NULL == n->u.condesc) return NULL; /* Table ovf */
 	n->type = CONNECTOR_type;
 	n->cost = 0.0;
 	return n;
@@ -921,6 +945,7 @@ static Exp * make_connector(Dictionary dict)
 		{
 			/* A simple, unidirectional connector. Just make that. */
 			n = make_dir_connector(dict, i);
+			if (NULL == n) return NULL;
 		}
 		else if (dict->token[i] == ANY_DIR)
 		{
@@ -929,8 +954,10 @@ static Exp * make_connector(Dictionary dict)
 			 * Make both a + and a - version, and or them together.  */
 			dict->token[i] = '+';
 			plu = make_dir_connector(dict, i);
+			if (NULL == plu) return NULL;
 			dict->token[i] = '-';
 			min = make_dir_connector(dict, i);
+			if (NULL == min) return NULL;
 
 			n = make_or_node(&dict->exp_list, plu, min);
 		}
@@ -943,7 +970,7 @@ static Exp * make_connector(Dictionary dict)
 
 	if (!link_advance(dict))
 	{
-		exp_free(n);
+		free(n);
 		return NULL;
 	}
 	return n;
@@ -962,6 +989,7 @@ void add_empty_word(Dictionary const dict, X_node *x)
 	Exp *zn, *an;
 	E_list *elist, *flist;
 	Exp_list eli = { NULL };
+	const char *ZZZ = string_set_add(EMPTY_CONNECTOR, dict->string_set);
 
 	/* The left-wall already has ZZZ-. The right-wall will not arrive here. */
 	if (MT_WALL == x->word->morpheme_type) return;
@@ -977,19 +1005,19 @@ void add_empty_word(Dictionary const dict, X_node *x)
 		/* zn points at {ZZZ+} */
 		zn = Exp_create(&eli);
 		zn->dir = '+';
-		zn->u.string = string_set_add(EMPTY_CONNECTOR, dict->string_set);
+		zn->u.condesc = condesc_add(&dict->contable, ZZZ);
 		zn->multi = false;
 		zn->type = CONNECTOR_type;
 		zn->cost = 0.0;
 		zn = make_optional_node(&eli, zn);
 
 		/* flist is plain-word-exp */
-		flist = (E_list *) xalloc(sizeof(E_list));
+		flist = (E_list *) malloc(sizeof(E_list));
 		flist->next = NULL;
 		flist->e = x->exp;
 
 		/* elist is {ZZZ+} , (plain-word-exp) */
-		elist = (E_list *) xalloc(sizeof(E_list));
+		elist = (E_list *) malloc(sizeof(E_list));
 		elist->next = flist;
 		elist->e = zn;
 
@@ -1012,10 +1040,9 @@ void add_empty_word(Dictionary const dict, X_node *x)
 static bool is_number(const char * str)
 {
 	if ('+' == str[0] || '-' == str[0]) str++;
-	if (strspn(str, "0123456789.") == strlen(str))
-		return true;
+	size_t numlen = strspn(str, "0123456789.");
 
-	return false;
+	return str[numlen] == '\0';
 }
 
 /* ======================================================================== */
@@ -1040,7 +1067,7 @@ static Exp * operator_exp(Dictionary dict, int type)
 	n->cost = 0.0;
 	elist = &first;
 	while((!is_equal(dict, ')')) && (!is_equal(dict, ']')) && (!is_equal(dict, '}'))) {
-		elist->next = (E_list *) xalloc(sizeof(E_list));
+		elist->next = (E_list *) malloc(sizeof(E_list));
 		elist = elist->next;
 		elist->next = NULL;
 		elist->e = expression(dict);
@@ -1407,20 +1434,32 @@ static Dict_node * dsw_vine_to_tree (Dict_node *root, int size)
 /* ======================================================================== */
 /**
  * Insert the new node into the dictionary below node n.
- * Give error message if the new element's string is already there.
- * Assumes that the "n" field of new is already set, and the left
- * and right fields of it are NULL.
+ * "newnode" left and right fields are NULL, and its string is already
+ * there.  If the string is already found in the dictionary, give an error
+ * message and effectively ignore it.
  *
  * The resulting tree is highly unbalanced. It needs to be rebalanced
  * before being used.  The DSW algo below is ideal for that.
  */
 Dict_node * insert_dict(Dictionary dict, Dict_node * n, Dict_node * newnode)
 {
-	int comp;
-
 	if (NULL == n) return newnode;
 
-	comp = dict_order_strict(newnode->string, n);
+	static Exp null_exp = { .type = AND_type, .u.l = NULL };
+	int comp = dict_order_strict(newnode->string, n);
+	if (0 == comp)
+	{
+		char t[80+MAX_TOKEN_LENGTH];
+		snprintf(t, sizeof(t),
+		         "Ignoring word \"%s\", which has been multiply defined "
+		         "(use verbosity=2 for more details): ",
+		         newnode->string);
+		dict_error(dict, t);
+		/* Too late to skip insertion - insert it with a null expression. */
+		newnode->exp = &null_exp;
+		comp = -1;
+	}
+
 	if (comp < 0)
 	{
 		if (NULL == n->left)
@@ -1429,10 +1468,8 @@ Dict_node * insert_dict(Dictionary dict, Dict_node * n, Dict_node * newnode)
 			return n;
 		}
 		n->left = insert_dict(dict, n->left, newnode);
-		return n;
-		/* return rebalance(n); Uncomment to get an AVL tree */
 	}
-	else if (comp > 0)
+	else
 	{
 		if (NULL == n->right)
 		{
@@ -1440,16 +1477,71 @@ Dict_node * insert_dict(Dictionary dict, Dict_node * n, Dict_node * newnode)
 			return n;
 		}
 		n->right = insert_dict(dict, n->right, newnode);
-		return n;
-		/* return rebalance(n); Uncomment to get an AVL tree */
+	}
+
+	return n;
+	/* return rebalance(n); Uncomment to get an AVL tree */
+}
+
+/**
+ * Find if a warning symbol exists in the currently suppress list.
+ * The warning symbols are constructed in a way that disallow overlap
+ * matching.
+ */
+static bool is_warning_suppressed(Dictionary dict, const char *warning_symbol)
+{
+	if (NULL == dict->suppress_warning) return false;
+	return (NULL != strstr(dict->suppress_warning, warning_symbol));
+}
+
+/**
+ * Remember the length_limit definitions in a list according to their order.
+ * The order is kept to allow later more specific definitions to override
+ * already applied ones.
+ */
+static void add_condesc_length_limit(Dictionary dict, Dict_node *dn,
+                                     int length_limit)
+{
+	length_limit_def_t *lld = malloc(sizeof(*lld));
+	lld->next = NULL;
+	lld->length_limit = length_limit;
+	lld->defexp = dn->exp;
+	lld->defword = dn->string;
+	*dict->contable.length_limit_def_next = lld;
+	dict->contable.length_limit_def_next = &lld->next;
+}
+
+static void insert_length_limit(Dictionary dict, Dict_node *dn)
+{
+	int length_limit;
+
+	if (0 == strcmp(UNLIMITED_CONNECTORS_WORD, dn->string))
+	{
+		length_limit = UNLIMITED_LEN;
 	}
 	else
+	if (0 == strncmp(LIMITED_CONNECTORS_WORD, dn->string,
+	                 sizeof(LIMITED_CONNECTORS_WORD)-1))
 	{
-		char t[256];
-		snprintf(t, 256, "The word \"%s\" has been multiply defined\n", newnode->string);
-		dict_error(dict, t);
-		return NULL;
+		char *endp;
+		length_limit =
+			(int)strtol(dn->string + sizeof(LIMITED_CONNECTORS_WORD)-1, &endp, 10);
+		if ((length_limit < 0) || (length_limit > MAX_SENTENCE) ||
+		  (('\0' != *endp) && (SUBSCRIPT_MARK != *endp)))
+		{
+			prt_error("Warning: Word \"%s\" found near line %d of %s.\n"
+					  "\tThis word should end with a number (1-%d).\n"
+					  "\tThis word will be ignored.",
+					  dn->string, dict->line_number, dict->name, MAX_SENTENCE);
+			return;
+		}
 	}
+	else return;
+
+	/* We cannot set the connectors length_limit yet because the
+	 * needed data structure is not defined yet. For now, just
+	 * remember the definitions in their order. */
+	add_condesc_length_limit(dict, dn, length_limit);
 }
 
 /**
@@ -1497,29 +1589,51 @@ void insert_list(Dictionary dict, Dict_node * p, int l)
 		        "\tWords ending \".Ix\" (x a number) are reserved for idioms.\n"
 		        "\tThis word will be ignored.",
 		        dn->string, dict->line_number, dict->name);
-		free_dict_node(dn);
-	}
-	else if ((dn_head = abridged_lookup_list(dict, dn->string)) != NULL)
-	{
-		char *u;
-		Dict_node *dnx;
-
-		u = strchr(dn->string, SUBSCRIPT_MARK);
-		if (u) *u = SUBSCRIPT_DOT;
-		prt_error("Warning: The word \"%s\" "
-		          "found near line %d of %s matches the following words:",
-	             dn->string, dict->line_number, dict->name);
-		for (dnx = dn_head; dnx != NULL; dnx = dnx->right) {
-			prt_error("\a\t%s", dnx->string);
-		}
-		prt_error("\a\n\tThis word will be ignored.\n");
-		file_free_lookup(dn_head);
-		free_dict_node(dn);
+		free(dn);
 	}
 	else
 	{
 		dict->root = insert_dict(dict, dict->root, dn);
+		insert_length_limit(dict, dn);
 		dict->num_entries++;
+
+		if ((verbosity_level(D_DICT+0) && !is_warning_suppressed(dict, DUP_BASE)) ||
+		    verbosity_level(D_SPEC+3))
+		{
+			/* Warn if there are words with a subscript that match a bare word. */
+			const char *sm = strchr(dn->string, SUBSCRIPT_MARK);
+			char *bareword;
+
+			if (NULL != sm)
+			{
+				bareword = strdupa(dn->string);
+				bareword[sm - dn->string] = '\0';
+			}
+			else
+			{
+				bareword = (char *)dn->string;
+			}
+
+			if ((dn_head = strict_lookup_list(dict, bareword)) != NULL)
+			{
+				bool match_found = false;
+				for (Dict_node *dnx = dn_head; dnx != NULL; dnx = dnx->right) {
+					if (0 != strcmp(dnx->string, dn->string))
+					{
+						if (!match_found)
+						{
+							prt_error("Warning: The word \"%s\" found near line "
+										 "%d of %s\n\t matches the following words:",
+										 dn->string, dict->line_number, dict->name);
+							match_found = true;
+						}
+						prt_error("\t%s", dnx->string);
+					}
+				}
+				if (match_found) prt_error("\n");
+				file_free_lookup(dn_head);
+			}
+		}
 	}
 
 	insert_list(dict, p, k);
@@ -1560,7 +1674,7 @@ static bool read_entry(Dictionary dict)
 				return false;
 			}
 		}
-		else if ((dict->token[0] == '#') && (0 == strcmp(dict->token, "#include")))
+		else if (0 == strcmp(dict->token, "#include"))
 		{
 			bool rc;
 			char* instr;
@@ -1569,14 +1683,14 @@ static bool read_entry(Dictionary dict)
 			bool save_is_special;
 			const char * save_input;
 			const char * save_pin;
-			char save_already_got_it;
+			int save_already_got_it;
 			int save_line_number;
 			size_t skip_slash;
 
 			if (!link_advance(dict)) goto syntax_error;
 
 			skip_slash          = ('/' == dict->token[0]) ? 1 : 0;
-			dict_name           = strdup(dict->token);
+			dict_name           = strdupa(dict->token);
 			save_name           = dict->name;
 			save_is_special     = dict->is_special;
 			save_input          = dict->input;
@@ -1609,7 +1723,6 @@ static bool read_entry(Dictionary dict)
 			dict->line_number    = save_line_number;
 
 			free(instr);
-			free(dict_name);
 			if (!rc) goto syntax_error;
 
 			/* when we return, point to the next entry */
@@ -1660,12 +1773,6 @@ static bool read_entry(Dictionary dict)
 		goto syntax_error;
 	}
 
-	/* pass the ; */
-	if (!link_advance(dict))
-	{
-		goto syntax_error;
-	}
-
 	/* At this point, dn points to a list of Dict_nodes connected by
 	 * their left pointers. These are to be inserted into the dictionary */
 	i = 0;
@@ -1675,6 +1782,19 @@ static bool read_entry(Dictionary dict)
 		i++;
 	}
 	dict->insert_entry(dict, dn, i);
+
+	if (dict->suppress_warning)
+	{
+		free((void *)dict->suppress_warning);
+		dict->suppress_warning = NULL;
+	}
+
+	/* pass the ; */
+	if (!link_advance(dict))
+	{
+		goto syntax_error;
+	}
+
 	return true;
 
 syntax_error:
