@@ -11,12 +11,14 @@
 /*************************************************************************/
 
 #include <string.h>
+
+#include "link-grammar/api-structures.h" // Sentence
 #include "connectors.h"
 #include "disjunct-utils.h"
 #include "print/print-util.h"
-#include "string-set.h"
 #include "utilities.h"
 #include "tokenize/tok-structures.h" // XXX TODO provide gword access methods!
+#include "tokenize/word-structures.h"
 
 /* Disjunct utilities ... */
 
@@ -35,6 +37,22 @@ void free_disjuncts(Disjunct *c)
 	}
 }
 
+void free_sentence_disjuncts(Sentence sent)
+{
+	if (NULL != sent->disjuncts_connectors_memblock)
+	{
+		free(sent->disjuncts_connectors_memblock);
+		sent->disjuncts_connectors_memblock = NULL;
+	}
+	else
+	{
+		for (WordIdx i = 0; i < sent->length; i++)
+		{
+			free_disjuncts(sent->word[i].d);
+		}
+	}
+}
+
 /**
  * Destructively catenates the two disjunct lists d1 followed by d2.
  * Doesn't change the contents of the disjuncts.
@@ -83,10 +101,10 @@ static inline unsigned int old_hash_disjunct(disjunct_dup_table *dt, Disjunct *
 	unsigned int i;
 	i = 0;
 	for (e = d->left ; e != NULL; e = e->next) {
-		i += string_hash(e->string);
+		i += e->desc->str_hash;
 	}
 	for (e = d->right ; e != NULL; e = e->next) {
-		i += string_hash(e->string);
+		i += e->desc->str_hash;
 	}
 	i += string_hash(d->word_string);
 	i += (i>>10);
@@ -94,13 +112,11 @@ static inline unsigned int old_hash_disjunct(disjunct_dup_table *dt, Disjunct *
 }
 
 /**
- * The connectors must be exactly equal.  A similar function
- * is connectors_equal_AND(), but that ignores priorities,
- * this does not.
+ * The connectors must be exactly equal.
  */
 static bool connectors_equal_prune(Connector *c1, Connector *c2)
 {
-	return string_set_cmp(c1->string, c2->string) && (c1->multi == c2->multi);
+	return c1->desc == c2->desc && (c1->multi == c2->multi);
 }
 
 /** returns TRUE if the disjuncts are exactly the same */
@@ -146,7 +162,7 @@ static Connector *connectors_dup(Connector *origc)
 
 	for (t = origc; t != NULL;  t = t->next)
 	{
-		newc = connector_new();
+		newc = connector_new(NULL, NULL);
 		*newc = *t;
 
 		prevc->next = newc;
@@ -184,6 +200,58 @@ Disjunct *disjuncts_dup(Disjunct *origd)
 	return head.next;
 }
 
+static Connector *pack_connectors_dup(Connector *origc, Connector **cblock)
+{
+	Connector head;
+	Connector *prevc = &head;
+	Connector *newc = &head;
+	Connector *t;
+	Connector *lcblock = *cblock; /* Optimization. */
+
+	for (t = origc; t != NULL;  t = t->next)
+	{
+		newc = lcblock++;
+		*newc = *t;
+
+		prevc->next = newc;
+		prevc = newc;
+	}
+	newc->next = NULL;
+
+	*cblock = lcblock;
+	return head.next;
+}
+
+/**
+ * Duplicate the given disjunct chain.
+ * If the argument is NULL, return NULL.
+ */
+Disjunct *pack_disjuncts_dup(Disjunct *origd, Disjunct **dblock, Connector **cblock)
+{
+	Disjunct head;
+	Disjunct *prevd = &head;
+	Disjunct *newd = &head;
+	Disjunct *t;
+	Disjunct *ldblock = *dblock; /* Optimization. */
+
+	for (t = origd; t != NULL; t = t->next)
+	{
+		newd = ldblock++;
+		newd->word_string = t->word_string;
+		newd->cost = t->cost;
+
+		newd->left = pack_connectors_dup(t->left, cblock);
+		newd->right = pack_connectors_dup(t->right, cblock);
+		newd->originating_gword = t->originating_gword;
+		prevd->next = newd;
+		prevd = newd;
+	}
+	newd->next = NULL;
+
+	*dblock = ldblock;
+	return head.next;
+}
+
 static disjunct_dup_table * disjunct_dup_table_new(size_t sz)
 {
 	size_t i;
@@ -215,7 +283,7 @@ GNUC_UNUSED static int gword_set_len(const gword_set *gl)
 
 /**
  * Return a new gword_set element, initialized from the given element.
- * @old_e Existing element.
+ * @param old_e Existing element.
  */
 static gword_set *gword_set_element_new(gword_set *old_e)
 {
@@ -249,8 +317,8 @@ static gword_set *gword_set_add(gword_set *gset, gword_set *ge)
  * This function is used for adding the gword pointers of an eliminated
  * disjunct to the ones of the kept disjuncts, with no duplicates.
  *
- * @kept gword_set of the kept disjunct.
- * @eliminated gword_set of the eliminated disjunct.
+ * @param kept gword_set of the kept disjunct.
+ * @param eliminated gword_set of the eliminated disjunct.
  * @return Use copy-on-write semantics - the gword_set of the kept disjunct
  * just gets returned if there is nothing to add to it. Else - a new gword
  * set is returned.
@@ -353,11 +421,11 @@ static void prt_con(Connector *c, dyn_str * p, char dir)
 
 	if (c->multi)
 	{
-		append_string(p, "@%s%c ", c->string, dir);
+		append_string(p, "@%s%c ", connector_string(c), dir);
 	}
 	else
 	{
-		append_string(p, "%s%c ", c->string, dir);
+		append_string(p, "%s%c ", connector_string(c), dir);
 	}
 }
 
