@@ -23,8 +23,6 @@
 #include "link-includes.h"
 #include "linkage.h"
 #include "lisjuncts.h"
-#include "post-process/post-process.h" // needed only for exfree_domain_names
-#include "post-process/pp-structures.h" // needed only for pp_info_s
 #include "sat-solver/sat-encoder.h"
 #include "string-set.h"
 #include "tokenize/wordgraph.h"
@@ -185,18 +183,8 @@ static void remap_linkages(Linkage lkg, const int *remap)
 
 			new_lnk->link_name = old_lnk->link_name;
 
-			/* Remap the pp_info, too. */
-			if (lkg->pp_info)
-				lkg->pp_info[j] = lkg->pp_info[i];
-
 			j++;
 		}
-		else
-		{
-			/* Whack this slot of pp_info. */
-			if (lkg->pp_info)
-				exfree_domain_names(&lkg->pp_info[i]);
-		}
 	}
 
 	lkg->num_links = j;
@@ -283,7 +271,7 @@ void compute_chosen_words(Sentence sent, Linkage linkage, Parse_Options opts)
 
 	Gword **nullblock_start = NULL; /* start of a null block, to be put in [] */
 	size_t nbsize = 0;              /* number of word in a null block */
-	Gword *unsplit_word = NULL;
+	Gword *sentence_word;
 
 	memset(show_word, 0, linkage->num_words * sizeof(*show_word));
 
@@ -298,7 +286,7 @@ void compute_chosen_words(Sentence sent, Linkage linkage, Parse_Options opts)
 		Gword **wgp;           /* wordgraph_path traversing pointer */
 
 		const char *t = NULL;  /* current word string */
-		bool nb_end;           /* current word is at end of a nullblock */
+		bool at_nullblock_end; /* current word is at end of a nullblock */
 		bool join_alt = false; /* morpheme-join this alternative */
 		char *s;
 		size_t l;
@@ -311,25 +299,27 @@ void compute_chosen_words(Sentence sent, Linkage linkage, Parse_Options opts)
 		w = lwg_path[i];
 		nw = lwg_path[i+1];
 		wgp = &lwg_path[i];
-		unsplit_word = w->unsplit_word;
+		sentence_word = wg_get_sentence_word(sent, w);
 
 		/* FIXME If the original word was capitalized in a capitalizable
 		 * position, the displayed null word may be its downcase version. */
 
 		if (NULL == cdj) /* a null word (the chosen disjunct was NULL) */
 		{
+			chosen_words[i] = NULL;
 			nbsize++;
 			if (NULL == nullblock_start) /* it starts a new null block */
 				nullblock_start = wgp;
 
-			nb_end = (NULL == nw) || (nw->unsplit_word != unsplit_word) ||
-				(MT_INFRASTRUCTURE == w->unsplit_word->morpheme_type);
+			at_nullblock_end = (NULL == nw) ||
+				(wg_get_sentence_word(sent, nw->unsplit_word) != sentence_word);
 
 			/* Accumulate null words in this alternative */
-			if (!nb_end && (NULL == cdjp[i+1]))
+			if (!at_nullblock_end && (NULL == cdjp[i+1]) &&
+			    ((w->morpheme_type == MT_PUNC) == (nw->morpheme_type == MT_PUNC)))
 			{
 				lgdebug(D_CCW, "Skipping word%zu cdjp=NULL#%zu, path %s\n",
-				         i, nbsize, lwg_path[i]->subword);
+				        i, nbsize, w->subword);
 				chosen_words[i] = NULL;
 				continue;
 			}
@@ -352,13 +342,13 @@ void compute_chosen_words(Sentence sent, Linkage linkage, Parse_Options opts)
 					lgdebug(D_CCW, "Combining null subwords");
 					/* Use alternative_id to check for start of alternative. */
 					if (((*nullblock_start)->alternative_id == *nullblock_start)
-					    && nb_end)
+					    && at_nullblock_end)
 					{
 						/* Case 2: A null unsplit_word (all-nulls alternative).*/
 						lgdebug(D_CCW, " (null alternative)\n");
-						t = unsplit_word->subword;
+						t = sentence_word->subword;
 
-						gwordlist_append(&n_lwg_path, unsplit_word);
+						gwordlist_append(&n_lwg_path, sentence_word);
 					}
 					else
 					{
@@ -411,7 +401,7 @@ void compute_chosen_words(Sentence sent, Linkage linkage, Parse_Options opts)
 				if (is_idiom_word(t))
 				{
 					s = strdupa(t);
-					sm = strrchr(s, SUBSCRIPT_MARK);
+					sm = strchr(s, SUBSCRIPT_MARK);
 					*sm = '\0';
 					t = string_set_add(s, sent->string_set);
 				}
@@ -431,6 +421,8 @@ void compute_chosen_words(Sentence sent, Linkage linkage, Parse_Options opts)
 
 					/* If the alternative contains morpheme subwords, mark it
 					 * for joining... */
+
+					const Gword *unsplit_word = w->unsplit_word;
 					for (wgaltp = wgp, j = i; NULL != *wgaltp; wgaltp++, j++)
 					{
 
@@ -481,7 +473,7 @@ void compute_chosen_words(Sentence sent, Linkage linkage, Parse_Options opts)
 							/* Cannot NULLify the word - we may have links to it. */
 							if (m != mcnt-1) chosen_words[i+m] = "";
 
-							sm =  strrchr(cdjp[i+m]->word_string, SUBSCRIPT_MARK);
+							sm =  strchr(cdjp[i+m]->word_string, SUBSCRIPT_MARK);
 
 							if (NULL != sm)
 							{
@@ -524,7 +516,7 @@ void compute_chosen_words(Sentence sent, Linkage linkage, Parse_Options opts)
 							 (SUBSCRIPT_MARK == join[join_len-1]))
 							join[join_len-1] = '\0';
 
-						gwordlist_append(&n_lwg_path, unsplit_word);
+						gwordlist_append(&n_lwg_path, sentence_word);
 						t = string_set_add(join, sent->string_set);
 						free(join);
 
@@ -750,13 +742,13 @@ const char * linkage_get_link_label(const Linkage linkage, LinkIdx index)
 const char * linkage_get_link_llabel(const Linkage linkage, LinkIdx index)
 {
 	if (!verify_link_index(linkage, index)) return NULL;
-	return linkage->link_array[index].lc->string;
+	return connector_string(linkage->link_array[index].lc);
 }
 
 const char * linkage_get_link_rlabel(const Linkage linkage, LinkIdx index)
 {
 	if (!verify_link_index(linkage, index)) return NULL;
-	return linkage->link_array[index].rc->string;
+	return connector_string(linkage->link_array[index].rc);
 }
 
 const char ** linkage_get_words(const Linkage linkage)
