@@ -37,12 +37,12 @@ struct Table_connector_s
 struct count_context_s
 {
 	fast_matcher_t *mchxt;
-	Word *  local_sent;
+	Sentence sent;
 	/* int     null_block; */ /* not used, always 1 */
 	bool    islands_ok;
 	bool    null_links;
 	bool    exhausted;
-	unsigned checktimer;  /* Avoid excess system calls */
+	unsigned int checktimer;  /* Avoid excess system calls */
 	int     table_size;
 	int     log2_table_size;
 	Table_connector ** table;
@@ -51,17 +51,6 @@ struct count_context_s
 
 static void free_table(count_context_t *ctxt)
 {
-	int i;
-	Table_connector *t, *x;
-
-	for (i=0; i<ctxt->table_size; i++)
-	{
-		for(t = ctxt->table[i]; t!= NULL; t=x)
-		{
-			x = t->next;
-			xfree((void *) t, sizeof(Table_connector));
-		}
-	}
 	xfree(ctxt->table, ctxt->table_size * sizeof(Table_connector*));
 	ctxt->table = NULL;
 	ctxt->table_size = 0;
@@ -87,13 +76,64 @@ static void init_table(count_context_t *ctxt, size_t sent_len)
 
 	/* Clamp at max 4*(1<<24) == 64 MBytes */
 	if (24 < shift) shift = 24;
+	lgdebug(+5, "Connector table size (1<<%u)*%zu\n", shift, sizeof(Table_connector));
 	ctxt->table_size = (1U << shift);
 	ctxt->log2_table_size = shift;
 	ctxt->table = (Table_connector**)
 		xalloc(ctxt->table_size * sizeof(Table_connector*));
 	memset(ctxt->table, 0, ctxt->table_size*sizeof(Table_connector*));
+
+
 }
 
+#ifdef DEBUG
+static int hit;
+static int miss;
+#define DEBUG_TABLE_STAT(x) x
+/**
+ * Provide data for insights on the effectively of the connector pair table.
+ * Hits, misses, chain length, number of elements with zero/nonzero counts.
+ */
+static void table_stat(count_context_t *ctxt, Sentence sent)
+{
+	int z = 0, nz = 0;
+	int c, N = 0;
+	const int sent_length = sent->length;
+	int *ww = alloca(sent_length*sent_length*sizeof(ww));
+	int *wc = alloca(sent_length*sent_length*sizeof(ww));
+
+	memset(ww, 0, sent_length*sent_length*sizeof(ww));
+	for (int i = 0; i < ctxt->table_size; i++)
+	{
+		c = 0;
+		Table_connector *t = ctxt->table[i];
+		if (t == NULL) N++;
+		for (; t != NULL; t = t->next)
+		{
+			if ((t->lw < 0) || (t->rw == sent_length)) continue;
+			c++;
+			if (hist_total(&t->count) == 0) z++;
+			else nz++;
+			ww[t->rw + sent_length * t->lw]++;
+			if (hist_total(&t->count)) wc[t->rw + sent_length * t->lw]++;
+		}
+		if (c != 0) printf("Connector table [%d] c=%d\n", i, c);
+	}
+
+	int wn = 0;
+	for (int i = 0; i < sent_length; i++)
+		for (int j = 0; j < sent_length; j++)
+		{
+			if (ww[i + sent_length * j]) wn++;
+			printf("WW %d %d = %d C=%d\n", i, j, ww[i + sent_length * j], wc[i + sent_length * j]);
+		}
+
+	printf("Connector table z=%d nz=%d N=%d hit=%d miss=%d wc=%d\n", z, nz, N, hit, miss, wn);
+}
+#else
+#define DEBUG_TABLE_STAT(x)
+#endif /* DEBUG */
+
 /**
  * Stores the value in the table.  Assumes it's not already there.
  */
@@ -105,7 +145,7 @@ static Table_connector * table_store(count_context_t *ctxt,
 	Table_connector *t, *n;
 	unsigned int h;
 
-	n = (Table_connector *) xalloc(sizeof(Table_connector));
+	n = pool_alloc(ctxt->sent->Table_connector_pool);
 	n->lw = lw; n->rw = rw; n->le = le; n->re = re; n->null_count = null_count;
 	h = pair_hash(ctxt->table_size, lw, rw, le, re, null_count);
 	t = ctxt->table[h];
@@ -128,8 +168,13 @@ find_table_pointer(count_context_t *ctxt,
 	for (; t != NULL; t = t->next) {
 		if ((t->lw == lw) && (t->rw == rw)
 		    && (t->le == le) && (t->re == re)
-		    && (t->null_count == null_count))  return t;
+		    && (t->null_count == null_count))
+		{
+			DEBUG_TABLE_STAT(hit++);
+			return t;
+		}
 	}
+	DEBUG_TABLE_STAT(miss++);
 
 	/* Create a new connector only if resources are exhausted.
 	 * (???) Huh? I guess we're in panic parse mode in that case.
@@ -162,10 +207,11 @@ Count_bin* table_lookup(count_context_t * ctxt,
 
 #define NO_COUNT -1
 #ifdef PERFORM_COUNT_HISTOGRAMMING
-Count_bin count_unknown = {.total = NO_COUNT};
+#define INIT_NO_COUNT {.total = NO_COUNT}
 #else
-Count_bin count_unknown = NO_COUNT;
+#define INIT_NO_COUNT NO_COUNT
 #endif
+Count_bin count_unknown = INIT_NO_COUNT;
 
 /**
  * psuedocount is used to check to see if a parse is even possible,
@@ -196,7 +242,7 @@ static int num_optional_words(count_context_t *ctxt, int w1, int w2)
 	int n = 0;
 
 	for (int w = w1+1; w < w2; w++)
-		if (ctxt->local_sent[w].optional) n++;
+		if (ctxt->sent->word[w].optional) n++;
 
 	return n;
 }
@@ -206,7 +252,12 @@ static int num_optional_words(count_context_t *ctxt, int w1, int w2)
 #endif
 
 #ifdef DO_COUNT_TRACE
-#define V(c) (!c?"(nil)":c->string)
+#define D_COUNT_TRACE 8
+#define LBLSZ 11
+#define TRACE_LABEL(l, do_count) \
+	(verbosity_level(D_COUNT_TRACE, "do_count") ? \
+	 prt_error("%-*s", LBLSZ, STRINGIFY(l)) : 0, do_count)
+#define V(c) (!c?"(nil)":connector_string(c))
 static Count_bin do_count1(int lineno, count_context_t *ctxt,
                           int lw, int rw,
                           Connector *le, Connector *re,
@@ -219,16 +270,20 @@ static Count_bin do_count(int lineno, count_context_t *ctxt,
 {
 	static int level;
 
-	if (!verbosity_level(8))
+	if (!verbosity_level(D_COUNT_TRACE))
 		return do_count1(lineno, ctxt, lw, rw, le, re, null_count);
 
 	Table_connector *t = find_table_pointer(ctxt, lw, rw, le, re, null_count);
+	char m_result[64] = "";
+	if (t != NULL)
+		snprintf(m_result, sizeof(m_result), "(M=%lld)", hist_total(&t->count));
 
 	level++;
-	prt_error("%*sdo_count%.*s:%d lw=%d rw=%d le=%s re=%s null_count=%d\n\\",
-		level*2, "", (!t)*3, "(R)", lineno, lw, rw, V(le), V(re), null_count);
+	prt_error("%*sdo_count%s:%d lw=%d rw=%d le=%s re=%s null_count=%d\n\\",
+		level*2, "", m_result, lineno, lw, rw, V(le), V(re), null_count);
 	Count_bin r = do_count1(lineno, ctxt, lw, rw, le, re, null_count);
-	prt_error("%*sreturn%.*s:%d=%lld\n", level*2, "", (!!t)*3, "(M)", lineno, r);
+	prt_error("%*sreturn%.*s:%d=%lld\n",
+	          LBLSZ+level*2, "", (!!t)*3, "(M)", lineno, hist_total(&r));
 	level--;
 
 	return r;
@@ -237,6 +292,7 @@ static Count_bin do_count(int lineno, count_context_t *ctxt,
 static Count_bin do_count1(int lineno,
 #define do_count(...) do_count(__LINE__, __VA_ARGS__)
 #else
+#define TRACE_LABEL(l, do_count) (do_count)
 static Count_bin do_count(
 #endif
                           count_context_t *ctxt,
@@ -255,8 +311,8 @@ static Count_bin do_count(
 
 	if (t) return t->count;
 
-	/* Create the table entry with a tentative null count of 0.
-	 * This count must be updated before we return. */
+	/* Create a table entry, to be updated with the found
+	 * linkage count before we return. */
 	t = table_store(ctxt, lw, rw, le, re, null_count);
 
 	int unparseable_len = rw-lw-1;
@@ -317,10 +373,10 @@ static Count_bin do_count(
 		 * of null words. */
 		t->count = zero;
 		w = lw + 1;
-		for (int opt = 0; opt <= !!ctxt->local_sent[w].optional; opt++)
+		for (int opt = 0; opt <= !!ctxt->sent->word[w].optional; opt++)
 		{
 			null_count += opt;
-			for (Disjunct *d = ctxt->local_sent[w].d; d != NULL; d = d->next)
+			for (Disjunct *d = ctxt->sent->word[w].d; d != NULL; d = d->next)
 			{
 				if (d->left == NULL)
 				{
@@ -364,7 +420,6 @@ static Count_bin do_count(
 #endif
 		for (size_t mle = mlb; get_match_list_element(mchxt, mle) != NULL; mle++)
 		{
-			int lnull_cnt, rnull_cnt;
 			Disjunct *d = get_match_list_element(mchxt, mle);
 			bool Lmatch = d->match_left;
 			bool Rmatch = d->match_right;
@@ -373,25 +428,26 @@ static Count_bin do_count(
 			assert(id == d->match_id, "Modified id (%d!=%d)", id, d->match_id);
 #endif
 
-			for (lnull_cnt = 0; lnull_cnt <= null_count; lnull_cnt++)
+			for (int lnull_cnt = 0; lnull_cnt <= null_count; lnull_cnt++)
 			{
+				int rnull_cnt = null_count - lnull_cnt;
+				/* Now lnull_cnt and rnull_cnt are the null-counts we're
+				 * requiring in those parts respectively. */
 				bool leftpcount = false;
 				bool rightpcount = false;
 
-				Count_bin l_any;
-				Count_bin l_cmulti = NO_COUNT;
-				Count_bin l_dmulti = NO_COUNT;
-				Count_bin l_dcmulti = NO_COUNT;
-				Count_bin l_bnr = NO_COUNT;
-				Count_bin r_any = NO_COUNT; /* gcc: may be used uninitialized */
-				Count_bin r_cmulti = NO_COUNT;
-				Count_bin r_dmulti = NO_COUNT;
-				Count_bin r_dcmulti = NO_COUNT;
-				Count_bin r_bnl = NO_COUNT;
-
-				rnull_cnt = null_count - lnull_cnt;
-				/* Now lnull_cnt and rnull_cnt are the costs we're assigning
-				 * to those parts respectively */
+				PRAGMA_MAYBE_UNINITIALIZED /* For old GCC versions */
+				Count_bin l_any;           /* Used only when leftpcount==true */
+				Count_bin r_any;           /* Used only when rightpcount==true */
+				PRAGMA_END
+				Count_bin l_cmulti = INIT_NO_COUNT;
+				Count_bin l_dmulti = INIT_NO_COUNT;
+				Count_bin l_dcmulti = INIT_NO_COUNT;
+				Count_bin l_bnr = INIT_NO_COUNT;
+				Count_bin r_cmulti = INIT_NO_COUNT;
+				Count_bin r_dmulti = INIT_NO_COUNT;
+				Count_bin r_dcmulti = INIT_NO_COUNT;
+				Count_bin r_bnl = INIT_NO_COUNT;
 
 				/* Now, we determine if (based on table only) we can see that
 				   the current range is not parsable. */
@@ -457,23 +513,26 @@ static Count_bin do_count(
 
 				if (!leftpcount && !rightpcount) continue;
 
+#define COUNT(c, do_count) \
+	{ c = TRACE_LABEL(c, do_count); }
 				if (!(leftpcount && rightpcount))
 				{
 					if (leftpcount)
 					{
 						/* Evaluate using the left match, but not the right. */
-						l_bnr = do_count(ctxt, w, rw, d->right, re, rnull_cnt);
+						COUNT(l_bnr, do_count(ctxt, w, rw, d->right, re, rnull_cnt));
 					}
 					else if (le == NULL)
 					{
 						/* Evaluate using the right match, but not the left. */
-						r_bnl = do_count(ctxt, lw, w, le, d->left, lnull_cnt);
+						COUNT(r_bnl, do_count(ctxt, lw, w, le, d->left, lnull_cnt));
 					}
 				}
 
 #define CACHE_COUNT(c, how_to_count, do_count) \
 { \
-	Count_bin count = (hist_total(&c) == NO_COUNT) ? do_count : hist_total(&c); \
+	Count_bin count = (hist_total(&c) == NO_COUNT) ? \
+		TRACE_LABEL(c, do_count) : c; \
 	how_to_count; \
 }
 			 /* If the pseudocounting above indicates one of the terms
@@ -522,13 +581,18 @@ static Count_bin do_count(
 
 					if (0 < hist_total(&rightcount))
 					{
-						/* Total number where links are used on both sides */
-						hist_muladd(&total, &leftcount, 0.0, &rightcount);
-
-						/* Evaluate using the right match, but not the left */
 						if (le == NULL)
+						{
+							/* Evaluate using the right match, but not the left */
 							CACHE_COUNT(r_bnl, hist_muladdv(&total, &rightcount, d->cost, count),
 								do_count(ctxt, lw, w, le, d->left, lnull_cnt));
+						}
+						else
+						{
+							/* Total number where links are used on both side.
+							 * Note that we don't have leftcount if le == NULL. */
+							hist_muladd(&total, &leftcount, 0.0, &rightcount);
+						}
 					}
 				}
 
@@ -592,7 +656,7 @@ Count_bin do_parse(Sentence sent,
 	ctxt->current_resources = opts->resources;
 	ctxt->exhausted = false;
 	ctxt->checktimer = 0;
-	ctxt->local_sent = sent->word;
+	ctxt->sent = sent;
 
 	/* consecutive blocks of this many words are considered as
 	 * one null link. */
@@ -602,24 +666,36 @@ Count_bin do_parse(Sentence sent,
 
 	hist = do_count(ctxt, -1, sent->length, NULL, NULL, null_count+1);
 
-	ctxt->local_sent = NULL;
-	ctxt->current_resources = NULL;
-	ctxt->checktimer = 0;
 	return hist;
 }
 
 /* sent_length is used only as a hint for the hash table size ... */
-count_context_t * alloc_count_context(size_t sent_length)
+count_context_t * alloc_count_context(Sentence sent)
 {
 	count_context_t *ctxt = (count_context_t *) xalloc (sizeof(count_context_t));
 	memset(ctxt, 0, sizeof(count_context_t));
 
-	init_table(ctxt, sent_length);
+	if (NULL != sent->Table_connector_pool)
+	{
+		pool_reuse(sent->Table_connector_pool);
+	}
+	else
+	{
+		sent->Table_connector_pool =
+			pool_new(__func__, "Table_connector",
+			         /*num_elements*/10240, sizeof(Table_connector),
+			         /*zero_out*/false, /*align*/false, /*exact*/false);
+	}
+
+	init_table(ctxt, sent->length);
 	return ctxt;
 }
 
-void free_count_context(count_context_t *ctxt)
+void free_count_context(count_context_t *ctxt, Sentence sent)
 {
+	if (NULL == ctxt) return;
+
+	DEBUG_TABLE_STAT(if (verbosity_level(D_SPEC+2)) table_stat(ctxt, sent));
 	free_table(ctxt);
 	xfree(ctxt, sizeof(count_context_t));
 }
