@@ -19,7 +19,7 @@ using std::vector;
 extern "C" {
 #include "sat-encoder.h"
 }
-#include "core/Solver.h"
+#include "minisat/core/Solver.h"
 #undef assert
 
 #include "sat-encoder.hpp"
@@ -467,7 +467,7 @@ Exp* SATEncoder::join_alternatives(int w)
   Exp* exp;
   E_list* or_list = NULL;;
   for (X_node* x = _sent->word[w].x; x != NULL; x = x->next) {
-    E_list* new_node = (E_list*) xalloc(sizeof(E_list));
+    E_list* new_node = (E_list*) malloc(sizeof(E_list));
     new_node->e = x->exp;
     new_node->next = NULL;
     if (or_list == NULL) {
@@ -479,7 +479,7 @@ Exp* SATEncoder::join_alternatives(int w)
       y->next = new_node;
     }
   }
-  exp = (Exp*) xalloc(sizeof(Exp));
+  exp = (Exp*) malloc(sizeof(Exp));
   exp->type = OR_type;
   exp->u.l = or_list;
   exp->cost = 0.0;
@@ -492,17 +492,17 @@ void SATEncoder::free_alternatives(Exp* exp)
   E_list *l = exp->u.l;
   while (l != NULL) {
     E_list* next = l->next;
-    xfree(l, sizeof(E_list));
+    free(l);
     l = next;
   }
-  xfree(exp, sizeof(exp));
+  free(exp);
 }
 
 
 void SATEncoder::generate_link_cw_ordinary_definition(size_t wi, int pi,
                                                       Exp* e, size_t wj)
 {
-  const char* Ci = e->u.string;
+  const char* Ci = e->u.condesc->string;
   char dir = e->dir;
   double cost = e->cost;
   Lit lhs = Lit(_variables->link_cw(wj, wi, pi, Ci));
@@ -519,12 +519,12 @@ void SATEncoder::generate_link_cw_ordinary_definition(size_t wi, int pi,
     if (dir == '+') {
       rhs.push(Lit(_variables->link_cost(wi, pi, Ci, e,
                                          (*i)->word, (*i)->position,
-                                         (*i)->connector.string,
+                                         connector_string(&(*i)->connector),
                                          (*i)->exp,
                                          cost + (*i)->cost)));
     } else if (dir == '-'){
       rhs.push(Lit(_variables->link((*i)->word, (*i)->position,
-                                    (*i)->connector.string,
+                                    connector_string(&(*i)->connector),
                                     (*i)->exp,
                                     wi, pi, Ci, e)));
     }
@@ -721,8 +721,8 @@ void SATEncoder::generate_conjunct_order_constraints(int w, Exp *e1, Exp* e2, in
         for (mw1i = mw1.begin(); mw1i != mw1.end(); mw1i++) {
           for (mw2i = mw2.begin(); mw2i != mw2.end(); mw2i++) {
             if (*mw1i >= *mw2i) {
-              clause[0] = ~Lit(_variables->link_cw(*mw1i, w, (*i)->position, (*i)->connector.string));
-              clause[1] = ~Lit(_variables->link_cw(*mw2i, w, (*j)->position, (*j)->connector.string));
+              clause[0] = ~Lit(_variables->link_cw(*mw1i, w, (*i)->position, connector_string(&(*i)->connector)));
+              clause[1] = ~Lit(_variables->link_cw(*mw2i, w, (*j)->position, connector_string(&(*j)->connector)));
               add_clause(clause);
             }
           }
@@ -766,8 +766,8 @@ void SATEncoder::generate_conjunct_order_constraints(int w, Exp *e1, Exp* e2, in
         for (mw1i = mw1.begin(); mw1i != mw1.end(); mw1i++) {
           for (mw2i = mw2.begin(); mw2i != mw2.end(); mw2i++) {
             if (*mw1i <= *mw2i) {
-              clause[0] = ~Lit(_variables->link_cw(*mw1i, w, (*i)->position, (*i)->connector.string));
-              clause[1] = ~Lit(_variables->link_cw(*mw2i, w, (*j)->position, (*j)->connector.string));
+              clause[0] = ~Lit(_variables->link_cw(*mw1i, w, (*i)->position, connector_string(&(*i)->connector)));
+              clause[1] = ~Lit(_variables->link_cw(*mw2i, w, (*j)->position, connector_string(&(*j)->connector)));
               add_clause(clause);
             }
           }
@@ -1421,8 +1421,7 @@ void SATEncoder::pp_prune()
  */
 Linkage SATEncoder::create_linkage()
 {
-  /* Using exalloc since this is external to the parser itself. */
-  Linkage linkage = (Linkage) exalloc(sizeof(struct Linkage_s));
+  Linkage linkage = (Linkage) malloc(sizeof(struct Linkage_s));
   memset(linkage, 0, sizeof(struct Linkage_s));
 
   partial_init_linkage(_sent, linkage, _sent->length);
@@ -1515,7 +1514,7 @@ Linkage SATEncoder::get_next_linkage()
   {
     _sent->num_linkages_alloced = _opts->linkage_limit;
     size_t nbytes = _sent->num_linkages_alloced * sizeof(struct Linkage_s);
-    _sent->lnkages = (Linkage)exalloc(nbytes);
+    _sent->lnkages = (Linkage) malloc(nbytes);
     _next_linkage_index = 0;
   }
   assert(_next_linkage_index<_sent->num_linkages_alloced, "_sent->lnkages ovl");
@@ -1523,7 +1522,7 @@ Linkage SATEncoder::get_next_linkage()
   Linkage lkg = &_sent->lnkages[_next_linkage_index];
   _next_linkage_index++;
   *lkg = *linkage;  /* copy en-mass */
-  exfree(linkage, sizeof(struct Linkage_s));
+  free(linkage);
 
   /* The link-parser code checks the next linkage for num_violations
    * (to save calls to linkage_create()). Allow for that practice. */
@@ -1531,10 +1530,10 @@ Linkage SATEncoder::get_next_linkage()
     lkg[1].lifo.N_violations = 0;
 
   // Perform the rest of the post-processing
-  PP_node *ppn = do_post_process(_sent->postprocessor, lkg, false);
-  if (NULL != ppn->violation) {
+  do_post_process(_sent->postprocessor, lkg, false);
+  if (NULL != _sent->postprocessor->violation) {
     lkg->lifo.N_violations++;
-    lkg->lifo.pp_violation_msg = ppn->violation;
+    lkg->lifo.pp_violation_msg = _sent->postprocessor->violation;
     lgdebug(+D_SAT, "Postprocessing error: %s\n", lkg->lifo.pp_violation_msg);
   } else {
     // XXX We cannot maintain num_valid_linkages, because it starts from
@@ -1542,14 +1541,11 @@ Linkage SATEncoder::get_next_linkage()
     // would report "Unique linkage".
     //_sent->num_valid_linkages++;
   }
-
-  build_type_array(_sent->postprocessor);
-  linkage_set_domain_names(_sent->postprocessor, lkg);
   post_process_free_data(&_sent->postprocessor->pp_data);
   linkage_score(lkg, _opts);
 
-  //if (NULL == ppn->violation && verbosity > 1)
-  //  _solver->printStats();
+  // if (NULL == _sent->postprocessor->violation && verbosity > 1)
+  //   _solver->printStats();
   return lkg;
 }
 
@@ -1575,7 +1571,7 @@ void SATEncoderConjunctionFreeSentences::determine_satisfaction(int w, char* nam
 void SATEncoderConjunctionFreeSentences::generate_satisfaction_for_connector(
     int wi, int pi, Exp *e, char* var)
 {
-  const char* Ci = e->u.string;
+  const char* Ci = e->u.condesc->string;
   char dir = e->dir;
   bool multi = e->multi;
   double cost = e->cost;
@@ -1637,7 +1633,7 @@ void SATEncoderConjunctionFreeSentences::generate_linked_definitions()
       for (c = w1_connectors.begin(); c != w1_connectors.end(); c++) {
         assert(c->word == w1, "Connector word must match");
         if (_word_tags[w2].match_possible(c->word, c->position)) {
-          rhs.push(Lit(_variables->link_cw(w2, c->word, c->position, c->connector.string)));
+          rhs.push(Lit(_variables->link_cw(w2, c->word, c->position, connector_string(&c->connector))));
         }
       }
 
@@ -1752,11 +1748,11 @@ void SATEncoder::generate_linked_min_max_planarity()
 
 Exp* SATEncoderConjunctionFreeSentences::PositionConnector2exp(const PositionConnector* pc)
 {
-    Exp* e = (Exp*)xalloc(sizeof(Exp));
+    Exp* e = (Exp*) malloc(sizeof(Exp));
     e->type = CONNECTOR_type;
     e->dir = pc->dir;
     e->multi = pc->connector.multi;
-    e->u.string = pc->connector.string;
+    e->u.condesc = (condesc_t *)pc->connector.desc; // FIXME - const
     e->cost = pc->cost;
 
     return e;
@@ -1769,8 +1765,8 @@ bool SATEncoderConjunctionFreeSentences::sat_extract_links(Linkage lkg)
 
   Exp **exp_word = (Exp **)alloca(_sent->length * sizeof(Exp));
   memset(exp_word, 0, _sent->length * sizeof(Exp));
-  const X_node **xnode_word = (const X_node **)alloca(_sent->length * sizeof(X_node));
-  memset(xnode_word, 0, _sent->length * sizeof(X_node));
+  const X_node **xnode_word = (const X_node **)alloca(_sent->length * sizeof(X_node *));
+  memset(xnode_word, 0, _sent->length * sizeof(X_node *));
 
   const std::vector<int>& link_variables = _variables->link_variables();
   std::vector<int>::const_iterator i;
@@ -1796,8 +1792,8 @@ bool SATEncoderConjunctionFreeSentences::sat_extract_links(Linkage lkg)
 
     // Allocate memory for the connectors, because they should persist
     // beyond the lifetime of the sat-solver data structures.
-    clink.lc = connector_new();
-    clink.rc = connector_new();
+    clink.lc = connector_new(NULL, NULL);
+    clink.rc = connector_new(NULL, NULL);
 
     *clink.lc = lpc->connector;
     *clink.rc = rpc->connector;
@@ -1850,7 +1846,10 @@ bool SATEncoderConjunctionFreeSentences::sat_extract_links(Linkage lkg)
       lgdebug(+0, "Warning: No expression for word %zu\n", wi);
     }
 
-    d = build_disjuncts_for_exp(de, xnode_word[wi]->string, UNLIMITED_LEN);
+#ifndef MAX_CONNECTOR_COST
+#define MAX_CONNECTOR_COST 1000.0f
+#endif
+    d = build_disjuncts_for_exp(de, xnode_word[wi]->string, MAX_CONNECTOR_COST, _opts);
     word_record_in_disjunct(xnode_word[wi]->word, d);
     lkg->chosen_disjuncts[wi] = d;
     free_Exp(de);
