@@ -81,6 +81,8 @@ void *alloca (size_t);
 #define vsnprintf _vsnprintf
 #endif
 
+#define HAVE__ALIGNED_MALLOC 1
+
 /* Avoid plenty of: warning C4090: 'function': different 'const' qualifiers.
  * This happens, for example, when the argument is "const void **". */
 #define free(x) free((void *)x)
@@ -171,6 +173,29 @@ typedef int locale_t;
 #define freelocale(l)
 #endif /* HAVE_LOCALE_T */
 
+#if HAVE__ALIGNED_MALLOC
+#define aligned_alloc(alignment, size) _aligned_malloc (size, alignment)
+#define aligned_free(p) _aligned_free(p)
+#undef HAVE_POSIX_MEMALIGN
+
+#elif HAVE_ALIGNED_ALLOC
+#define aligned_free(p) free(p)
+#undef HAVE_POSIX_MEMALIGN
+
+#elif HAVE_POSIX_MEMALIGN
+/* aligned_alloc() emulation will be defined in utilities.c. */
+void *aligned_alloc(size_t alignment, size_t size);
+#define aligned_free(p) free(p)
+
+#else
+/* Fallback to just malloc(), as alignment is not critical here. */
+#define NO_ALIGNED_MALLOC /* For generating a warning in utilities.c. */
+#define aligned_alloc(alignment, size) malloc(size)
+#define aligned_free(p) free(p)
+#endif /* HAVE__ALIGNED_MALLOC */
+
+#define ALIGN(size, alignment) (((size)+(alignment-1))&~(alignment-1))
+
 #define STR(x) #x
 #define STRINGIFY(x) STR(x)
 
@@ -205,8 +230,8 @@ static inline char *_strndupa3(char *new_s, const char *s, size_t n)
 #define BUILD_ASSERT_OR_ZERO(cond) (sizeof(char [1 - 2*!(cond)]) - 1)
 #if !defined(ARRAY_SIZE)
 /**
- * ARRAY_SIZE - get the number of elements in a visible array
- * @arr: the array whose size you want.
+ * ARRAY_SIZE: Get the number of elements in a visible array
+ * @param arr The array whose size you want.
  *
  * This does not work on pointers, or arrays declared as [], or
  * function parameters.  With correct compiler support, such usage
@@ -225,18 +250,21 @@ static inline char *_strndupa3(char *new_s, const char *s, size_t n)
 #endif
 #endif /* !defined(ARRAY_SIZE) */
 
+/* The GCC version we need must be >= 4.7, because it has to
+ * support C11. So it already supports all the features below. */
+
 /* Optimizations etc. that only gcc understands */
-#if __GNUC__ > 2
+#if __GNUC__
+#define GCC_DIAGNOSTIC
+#define UNREACHABLE(x) (__extension__ ({if (x) __builtin_unreachable();}))
 #define GNUC_MALLOC __attribute__ ((malloc))
 #define GNUC_UNUSED __attribute__ ((unused))
 #else
+#define UNREACHABLE(x)
 #define GNUC_MALLOC
 #define GNUC_UNUSED
 #endif
 
-#if ((__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 7)) || __clang__)
-#define GCC_DIAGNOSTIC
-#endif
 
 /* Apply a pragma to a specific code section only.
  * XXX According to the GCC docs, we cannot use here something like
@@ -441,6 +469,7 @@ typedef struct
 dyn_str* dyn_str_new(void);
 void dyn_str_delete(dyn_str*);
 void dyn_strcat(dyn_str*, const char*);
+void dyn_trimback(dyn_str*);
 char * dyn_str_take(dyn_str*);
 const char * dyn_str_value(dyn_str*);
 
@@ -471,9 +500,9 @@ bool try_locale(const char *);
 /**
  * Returns the smallest power of two that is at least i and at least 1
  */
-static inline unsigned int next_power_of_two_up(unsigned int i)
+static inline size_t next_power_of_two_up(size_t i)
 {
-	unsigned int j=1;
+	size_t j=1;
 	while (j<i) j <<= 1;
 	return j;
 }
