@@ -274,6 +274,7 @@ static int x_issue_special_command(char * line, Command_Options *copts, Dictiona
 				if ((Bool == as[i].param_type) || Int == as[i].param_type)
 				{
 					printf("%5d", ival(as[i]));
+					if (-1 == ival(as[i])) printf(" (Unlimited)");
 				}
 				else
 				if (String == as[i].param_type)
@@ -341,6 +342,8 @@ static int x_issue_special_command(char * line, Command_Options *copts, Dictiona
 #ifdef USE_REGEX_TOKENIZER
 	if (s[0] == '/')
 	{
+		/* experimental code -- ignore nested extern warning for now */
+		extern int regex_tokenizer_test(Dictionary, const char *);
 		int rc = regex_tokenizer_test(dict, s+1);
 		if (0 != rc) printf("regex_tokenizer_test: rc %d\n", rc);
 		return 0;
