@@ -487,7 +487,7 @@ static void setup_panic_parse_options(Parse_Options opts)
 	parse_options_set_max_parse_time(opts, 60);
 	parse_options_set_islands_ok(opts, false);
 	parse_options_set_short_length(opts, 12);
-	parse_options_set_all_short_connectors(opts, 1);
+	parse_options_set_all_short_connectors(opts, true);
 	parse_options_set_linkage_limit(opts, 100);
 	parse_options_set_spell_guess(opts, 0);
 }
@@ -684,7 +684,7 @@ int main(int argc, char * argv[])
 		Sentence sent = NULL;
 
 		/* Make sure stderr is shown even when MSVC binary runs under
-		 * Cygwin/MSYS pty (in that case it is fully buffered(!). */
+		 * Cygwin/MSYS pty (in that case it is fully buffered(!)). */
 		fflush(stderr);
 
 		verbosity = parse_options_get_verbosity(opts);
@@ -897,7 +897,7 @@ int main(int argc, char * argv[])
 					parse_options_get_use_sat_parser(opts));
 				parse_options_reset_resources(copts->panic_opts);
 				parse_options_set_verbosity(copts->panic_opts, verbosity);
-				num_linkages = sentence_parse(sent, copts->panic_opts);
+				(void)sentence_parse(sent, copts->panic_opts);
 				if (verbosity > 0)
 				{
 					if (parse_options_timer_expired(copts->panic_opts))
